{% extends 'index.html' %}

{% block content %}
<div id="fake-mouse">
    <span><i class="fas fa-mouse-pointer"></i></span>
</div>
{{ super() }}
{% endblock %}

{% block js %}
{{ super() }}
{% if term is not none %}
    <script src="{{ url_for('static', filename='js/typed.min.js') }}"></script>
    <script>
        $(document).ready(function() {
            lmzify();
        });
    </script>
{% endif %}
{% endblock %}
