# Copyright 2018 BMW Car IT GmbH
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import subprocess

import click
from flask.cli import current_app, with_appcontext


@click.command()
@click.argument("dst", type=click.Path(exists=True))
@with_appcontext
def collectstatic(dst):
    for blueprint in current_app.iter_blueprints():
        subprocess.run(["cp", "-rv", blueprint.static_folder, dst])
    subprocess.run(["cp", "-rv", current_app.static_folder, dst])
